Sign

Enum Sign 

#[repr(u8)]
pub enum Sign { Plus = 0, Minus = 1, }
A Sign represents sign associated with a decimal number.

Variants



Plus = 0

Positive: plus “+” or no sign.



Minus = 1

Negative: minus “-” sign.

Implementations

Source

impl Sign

pub const fn default() -> Self

Returns the default Plus.

pub const fn is_negative(self) -> bool

Returns true if sign is negative, and false otherwise.

pub const fn eq(&self, other: &Self) -> bool

Tests for self and other signs to be equal, and is used by == operator.

pub const fn not(self) -> Self

Invert Sign value.

Example
use fastnum::decimal::Sign;

assert_eq!(Sign::Plus.not(), Sign::Minus);
assert_eq!(Sign::Minus.not(), Sign::Plus);
pub const fn mul(self, rhs: Self) -> Self

Sign “multiplication”.

pub const fn div(self, rhs: Self) -> Self

Sign “division”.