pub struct SignalsTraps(/* private fields */);Expand description
SignalsTraps
SignalsTraps is a list of set trap enablers for signals.
When a signal’s trap enabler is set, the condition causes panic! in debug
mode.
Implementations