UnsignedDecimal

Struct UnsignedDecimal 

pub struct UnsignedDecimal<const N: usize>(/* private fields */);
Unsigned Decimal

Generic unsigned N-bits decimal number.

Implementations

Source

impl<const N: usize> UnsignedDecimal<N>

pub const fn from_u8(n: u8) -> Self

Converts u8 to Decimal.

pub const fn from_u16(n: u16) -> Self

Converts u16 to Decimal.

pub const fn from_u32(n: u32) -> Self

Converts u32 to Decimal.

pub const fn from_u64(n: u64) -> Self

Converts u64 to Decimal.

pub const fn from_u128(n: u128) -> Result<Self, ParseError>

Converts u128 to Decimal.

pub const fn from_usize(n: usize) -> Self

Converts usize to Decimal.

Source

impl<const N: usize> UnsignedDecimal<N>

pub const fn from_i8(int: i8) -> Result<Self, ParseError>

Try converts i8 to UnsignedDecimal.

pub const fn from_i16(int: i16) -> Result<Self, ParseError>

Try converts i16 to UnsignedDecimal.

pub const fn from_i32(int: i32) -> Result<Self, ParseError>

Try converts i32 to UnsignedDecimal.

pub const fn from_i64(int: i64) -> Result<Self, ParseError>

Try converts i64 to UnsignedDecimal.

pub const fn from_i128(int: i128) -> Result<Self, ParseError>

Try converts i128 to UnsignedDecimal.

pub const fn from_isize(int: isize) -> Result<Self, ParseError>

Try converts isize to UnsignedDecimal.

Source

impl<const N: usize> UnsignedDecimal<N>

pub const fn from_f32(n: f32) -> Result<Self, ParseError>

Try converts f32 to UnsignedDecimal.

pub const fn from_f64(n: f64) -> Result<Self, ParseError>

Try converts f64 to UnsignedDecimal.

Source

impl<const N: usize> UnsignedDecimal<N>

pub const fn to_u8(self) -> Result<u8, ParseError>

Try converts UnsignedDecimal into u8.

pub const fn to_u16(self) -> Result<u16, ParseError>

Try converts UnsignedDecimal into u16.

pub const fn to_u32(self) -> Result<u32, ParseError>

Try converts UnsignedDecimal into u32.

pub const fn to_u64(self) -> Result<u64, ParseError>

Try converts UnsignedDecimal into u64.

pub const fn to_u128(self) -> Result<u128, ParseError>

Try converts UnsignedDecimal into u128.

pub const fn to_usize(self) -> Result<usize, ParseError>

Try converts UnsignedDecimal into usize.

pub const fn to_i8(self) -> Result<i8, ParseError>

Try converts UnsignedDecimal into i8.

pub const fn to_i16(self) -> Result<i16, ParseError>

Try converts UnsignedDecimal into i16.

pub const fn to_i32(self) -> Result<i32, ParseError>

Try converts UnsignedDecimal into i32.

pub const fn to_i64(self) -> Result<i64, ParseError>

Try converts UnsignedDecimal into i64.

pub const fn to_i128(self) -> Result<i128, ParseError>

Try converts UnsignedDecimal into i128.

pub const fn to_isize(self) -> Result<isize, ParseError>

Try converts UnsignedDecimal into isize.

Source

impl<const N: usize> UnsignedDecimal<N>

pub const fn to_f32(self) -> f32

Converts UnsignedDecimal into f32.

pub const fn to_f64(self) -> f64

Converts UnsignedDecimal into f64.

Source

impl<const N: usize> UnsignedDecimal<N>

pub const NAN: Self

Not A Number value. More about NaN.

pub const INFINITY: Self

Infinity (∞). More about ±Infinity.

pub const MIN: Self = Self::ZERO

The smallest value that can be represented by this decimal type (0).

pub const MAX: Self

The largest value that can be represented by this decimal type (2N − 1)×1032’768.

pub const MIN_POSITIVE: Self

The smallest positive, normalized value that this type can represent.

pub const EPSILON: Self

Machine epsilon value.

This is the difference between 1.0 and the next larger representable number.

pub const ZERO: Self

The value of 0 represented by this decimal type.

pub const ONE: Self

The value of 1 represented by this decimal type.

pub const TWO: Self

The value of 2 represented by this decimal type.

pub const THREE: Self

The value of 3 represented by this decimal type.

pub const FOUR: Self

The value of 4 represented by this decimal type.

pub const FIVE: Self

The value of 5 represented by this decimal type.

pub const SIX: Self

The value of 6 represented by this decimal type.

pub const SEVEN: Self

The value of 7 represented by this decimal type.

pub const EIGHT: Self

The value of 8 represented by this decimal type.

pub const NINE: Self

The value of 9 represented by this decimal type.

pub const TEN: Self

The value of 10 represented by this decimal type.

pub const E: Self

Euler’s number (e).

pub const PI: Self

Archimedes’ constant (π).

pub const TAU: Self

The full circle constant (τ)

Equal to 2π.

pub const FRAC_1_PI: Self

1/π.

pub const FRAC_2_PI: Self

2/π.

pub const FRAC_PI_2: Self

π/2.

pub const FRAC_PI_3: Self

π/3.

pub const FRAC_PI_4: Self

π/4.

pub const FRAC_PI_6: Self

π/6.

pub const FRAC_PI_8: Self

π/8.

pub const FRAC_2_SQRT_PI: Self

2/sqrt(π).

pub const LN_2: Self

ln(2).

pub const LN_10: Self

ln(10).

pub const LOG2_E: Self

log2(e).

pub const LOG10_E: Self

log10(e).

pub const SQRT_2: Self

sqrt(2).

pub const FRAC_1_SQRT_2: Self

1/sqrt(2).

pub const LOG10_2: Self

log10(2).

pub const LOG2_10: Self

log2(10).

Source

impl<const N: usize> UnsignedDecimal<N>

pub const fn from_parts(digits: UInt<N>, exp: i32, ctx: Context) -> Self

Creates and initializes unsigned decimal from parts.

Examples
use fastnum::{*, decimal::*};

assert_eq!(UD256::from_parts(u256!(12345), -4, Context::default()), udec256!(1.2345));
pub const fn from_str(s: &str, ctx: Context) -> Result<Self, ParseError>

Creates and initializes an unsigned decimal from string.

Examples
use fastnum::{*, decimal::*};

assert_eq!(UD256::from_str("1.2345", Context::default()), Ok(udec256!(1.2345)));
assert_eq!(UD256::from_str("-1.2345", Context::default()), Err(ParseError::Signed));
pub const fn parse_str(s: &str, ctx: Context) -> Self

Parse an unsigned decimal from string.

Panics

This function will panic if UnsignedDecimal<N> can’t be constructed from a given string.

Examples

Basic usage:

use fastnum::{*, decimal::*};

assert_eq!(UD256::parse_str("1.2345", Context::default()), udec256!(1.2345));

Should panic:

use fastnum::{*, decimal::*};

let _ = UD256::parse_str("-1.2345", Context::default());
pub const fn digits(&self) -> UInt<N>

Returns the internal big integer, representing the Coefficient of a given UnsignedDecimal, including significant trailing zeros.

Examples
use fastnum::{udec256, u256};

let a = udec256!(123.45);
assert_eq!(a.digits(), u256!(12345));

let b = udec256!(1.0);
assert_eq!(b.digits(), u256!(10));
pub const fn digits_count(&self) -> usize

Returns the count of digits in the non-scaled integer representation

pub const fn fractional_digits_count(&self) -> i16

Returns the scale of the UnsignedDecimal, the total number of digits to the right of the decimal point (including insignificant leading zeros).

Examples
use fastnum::udec256;

let a = udec256!(12345);  // No fractional part
let b = udec256!(123.45);  // Fractional part
let c = udec256!(0.0000012345);  // Completely fractional part
let d = udec256!(500000000);  // No fractional part
let e = udec256!(5e9);  // Negative-fractional part

assert_eq!(a.fractional_digits_count(), 0);
assert_eq!(b.fractional_digits_count(), 2);
assert_eq!(c.fractional_digits_count(), 10);
assert_eq!(d.fractional_digits_count(), 0);
assert_eq!(e.fractional_digits_count(), -9);
pub const fn is_zero(&self) -> bool

Return if the referenced unsigned decimal is zero.

Examples
use fastnum::{udec256};

let a = udec256!(0);
assert!(a.is_zero());

let b = udec256!(0.0);
assert!(b.is_zero());

let c = udec256!(0.00);
assert!(c.is_zero());

let d = udec256!(0.1);
assert!(!d.is_zero());
pub const fn is_one(&self) -> bool

Return if the referenced unsigned decimal is strictly Self::ONE.

Examples
use fastnum::{udec256};

let a = udec256!(1);
assert!(a.is_one());

let b = udec256!(10e-1);
assert!(!b.is_one());
pub const fn is_op_div_by_zero(&self) -> bool

Returns true if the given decimal number is the result of division by zero and false otherwise.

Examples
use fastnum::{*, decimal::*};

let ctx = Context::default().without_traps();
let res = udec256!(1.0).with_ctx(ctx) / udec256!(0).with_ctx(ctx);

assert!(res.is_op_div_by_zero());

More about OP_DIV_BY_ZERO signal.

pub const fn is_op_invalid(&self) -> bool

Return true if the argument has Signals::OP_INVALID signal flag, and false otherwise.

pub const fn is_op_subnormal(&self) -> bool

Return true if the argument has Signals::OP_SUBNORMAL signal flag, and false otherwise.

pub const fn is_op_inexact(&self) -> bool

Return true if the argument has Signals::OP_INEXACT signal flag, and false otherwise.

pub const fn is_op_rounded(&self) -> bool

Return true if the argument has Signals::OP_ROUNDED signal flag, and false otherwise.

pub const fn is_op_clamped(&self) -> bool

Return true if the argument has Signals::OP_CLAMPED signal flag, and false otherwise.

pub const fn is_op_overflow(&self) -> bool

Return true if the argument has Signals::OP_OVERFLOW signal flag, and false otherwise.

pub const fn is_op_underflow(&self) -> bool

Return true if the argument has Signals::OP_UNDERFLOW signal flag, and false otherwise.

pub const fn is_op_ok(&self) -> bool

Return true if the argument has no signal flags, and false otherwise.

pub const fn op_signals(&self) -> Signals

Return the signaling block of given unsigned decimal.

pub const fn classify(&self) -> FpCategory

Return the decimal category of the number. If only one property is going to be tested, it is generally faster to use the specific predicate instead.

Examples
use core::num::FpCategory;
use fastnum::{udec256, UD256};

let num = udec256!(12.4);
let inf = UD256::INFINITY;

assert_eq!(num.classify(), FpCategory::Normal);
assert_eq!(inf.classify(), FpCategory::Infinite);
pub const fn is_normal(self) -> bool

Return true if the number is neither zero, Infinity, subnormal, or NaN and false otherwise.

Examples
use fastnum::*;

let num = udec256!(12.4);
let subnormal = udec256!(1E-30000) / udec256!(1E2768);
let inf = UD256::INFINITY;
let nan = UD256::NAN;
let zero = UD256::ZERO;

assert!(num.is_normal());

assert!(!zero.is_normal());
assert!(!nan.is_normal());
assert!(!nan.is_normal());
assert!(!subnormal.is_normal());
pub const fn is_subnormal(self) -> bool

Return true if the number is subnormal and false otherwise.

Examples
use fastnum::*;

let num = udec256!(12.4);
let subnormal = udec256!(1E-30000) / udec256!(1E2768);
let inf = UD256::INFINITY;
let nan = UD256::NAN;
let zero = UD256::ZERO;

assert!(subnormal.is_subnormal());

assert!(!num.is_subnormal());
assert!(!zero.is_subnormal());
assert!(!nan.is_subnormal());
assert!(!nan.is_subnormal());
pub const fn is_finite(self) -> bool

Return true if this number is neither Infinity nor NaN and false otherwise.

Examples
use fastnum::{UD256, udec256};

let d = udec256!(7.0);
let inf = UD256::INFINITY;
let nan = UD256::NAN;

assert!(d.is_finite());

assert!(!nan.is_finite());
assert!(!inf.is_finite());
pub const fn is_infinite(self) -> bool

Return true if this value is Infinity and false otherwise.

Examples
use fastnum::{UD256, udec256};

let d = udec256!(7.0);
let inf = UD256::INFINITY;
let nan = UD256::NAN;

assert!(inf.is_infinite());

assert!(!d.is_infinite());
assert!(!nan.is_infinite());
pub const fn is_nan(self) -> bool

Return true if this value is NaN and false otherwise.

Examples
use fastnum::{UD256, udec256};

let nan = UD256::NAN;
let d = udec256!(7.0);

assert!(nan.is_nan());
assert!(!d.is_nan());
pub const fn with_ctx(self, ctx: Context) -> Self

Apply new Context to the given decimal number.

Returns a copy of the value with the provided context applied.

This method updates the operational context (including the rounding mode and other contextual parameters) used by subsequent operations that may round or clamp the value (e.g., add, sub, mul, div, round, rescale, quantize, etc.).

Important:

  • The change is local to the returned value and does not affect other values.
  • If you ignore the returned value, the context update is lost.
  • If the value currently carries extra precision, that extra precision is reconciled immediately with the new context: it is rounded using the context’s rounding mode (and may be clamped as dictated by the context).
  • If the current value already has signaling flags set (e.g., INEXACT and the new context enables traps for those signals, applying this method may trigger the corresponding traps immediately, which can result in a panic (depending on the build/configuration).
Panics:
debug mode

This method will panic if

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

let ctx = decimal::Context::default().without_traps();
let a = udec256!(1).with_ctx(ctx);
let b = udec256!(0).with_ctx(ctx);

// No panic! We can divide by zero!
let c = a / b;
assert!(c.is_infinite());
assert!(c.is_op_div_by_zero());

See also:

pub const fn with_rounding_mode(self, rm: RoundingMode) -> Self

Apply new RoundingMode to the given decimal number.

Returns a copy of the value with an updated rounding mode in its context.

This method generally does not immediately change the mathematical value; it only sets the rounding rule that will be used by subsequent operations that may round (e.g., add, sub, mul, div, round, rescale, quantize, conversions, etc.).

Important:

  • The change is local to the returned value and does not affect other values.
  • If you ignore the returned value, the rounding mode update is lost.
  • If the value currently carries extra precision, that extra precision is rounded using the newly provided rounding mode at the moment this method is applied. This ensures internal consistency of the stored representation with the new rounding rule.
Panics:
debug mode

This method will panic if possible extra precision rounding operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

let a = udec256!(1).with_rounding_mode(decimal::RoundingMode::No);
let b = udec256!(3).with_rounding_mode(decimal::RoundingMode::No);
let c = udec256!(6).with_rounding_mode(decimal::RoundingMode::No);

assert_eq!(((a / b) * c).with_rounding_mode(decimal::RoundingMode::HalfUp), udec256!(2));

See also:

pub const fn quantum(exp: i32, ctx: Context) -> Self

The quantum of a finite number is given by: 1 × 10exp. This is the value of a unit in the least significant position of the coefficient of a finite number.

Examples
use fastnum::{*, decimal::*};

let ctx = Context::default();

assert_eq!(UD256::quantum(0, ctx), udec256!(1));
assert_eq!(UD256::quantum(-0, ctx), udec256!(1));
assert_eq!(UD256::quantum(-3, ctx), udec256!(0.001));
assert_eq!(UD256::quantum(3, ctx), udec256!(1000));
pub const fn reduce(self) -> Self

Reduces a decimal number to its shortest (coefficient) form shifting all significant trailing zeros into the exponent.

Examples
use fastnum::{udec256, u256, decimal::Context};

let a = udec256!(1234500);
assert_eq!(a.digits(), u256!(1234500));
assert_eq!(a.fractional_digits_count(), 0);

let b = a.reduce();
assert_eq!(b.digits(), u256!(12345));
assert_eq!(b.fractional_digits_count(), -2);
pub const fn neg(self) -> Decimal<N>

Invert sign of the given unsigned decimal.

Examples
use fastnum::*;

assert_eq!(udec256!(1.0).neg(), dec256!(-1.0));
pub const fn eq(&self, other: &Self) -> bool

Tests for self and other values to be equal, and is used by == operator.

pub const fn ne(&self, other: &Self) -> bool

Tests for self and other values to be equal, and is used by == operator.

pub const fn max(self, other: Self) -> Self

Compares and returns the maximum of two unsigned decimal values.

Returns the second argument if the comparison determines them to be equal.

Examples
use fastnum::{udec256};

assert_eq!(udec256!(1).max(udec256!(2)), udec256!(2));
assert_eq!(udec256!(2).max(udec256!(2)), udec256!(2));
pub const fn min(self, other: Self) -> Self

Compares and returns the minimum of two undecimal values.

Returns the first argument if the comparison determines them to be equal.

Examples
use fastnum::udec256;

assert_eq!(udec256!(1).min(udec256!(2)), udec256!(1));
assert_eq!(udec256!(2).min(udec256!(2)), udec256!(2));
pub const fn clamp(self, min: Self, max: Self) -> Self

Restrict an unsigned decimal value to a certain interval.

Returns max if self is greater than max, and min if self is less than min. Otherwise, this returns self.

Panics

Panics if min > max.

Examples
use fastnum::udec256;

assert_eq!(udec256!(0).clamp(udec256!(3), udec256!(5)), udec256!(3));
assert_eq!(udec256!(3).clamp(udec256!(1), udec256!(5)), udec256!(3));
assert_eq!(udec256!(6).clamp(udec256!(1), udec256!(5)), udec256!(5));
pub const fn lt(&self, other: &Self) -> bool

Tests unsigned decimal self less than other and is used by the < operator.

Examples
use fastnum::udec256;

assert_eq!(udec256!(1.0).lt(&udec256!(1.0)), false);
assert_eq!(udec256!(1.0).lt(&udec256!(2.0)), true);
assert_eq!(udec256!(2.0).lt(&udec256!(1.0)), false);
pub const fn le(&self, other: &Self) -> bool

Tests unsigned decimal self less than or equal to other and is used by the <= operator.

Examples
use fastnum::udec256;

assert_eq!(udec256!(1.0).le(&udec256!(1.0)), true);
assert_eq!(udec256!(1.0).le(&udec256!(2.0)), true);
assert_eq!(udec256!(2.0).le(&udec256!(1.0)), false);
pub const fn gt(&self, other: &Self) -> bool

Tests unsigned decimal self greater than other and is used by the > operator.

Examples
use fastnum::udec256;

assert_eq!(udec256!(1.0).gt(&udec256!(1.0)), false);
assert_eq!(udec256!(1.0).gt(&udec256!(2.0)), false);
assert_eq!(udec256!(2.0).gt(&udec256!(1.0)), true);
pub const fn ge(&self, other: &Self) -> bool

Tests unsigned decimal self greater than or equal to other and is used by the >= operator.

Examples
use fastnum::udec256;

assert_eq!(udec256!(1.0).ge(&udec256!(1.0)), true);
assert_eq!(udec256!(1.0).ge(&udec256!(2.0)), false);
assert_eq!(udec256!(2.0).ge(&udec256!(1.0)), true);
pub const fn cmp(&self, other: &Self) -> Ordering

This method returns an Ordering between self and other.

By convention, self.cmp(&other) returns the ordering matching the expression self <operator> other if true.

Examples
use fastnum::udec256;
use std::cmp::Ordering;

assert_eq!(udec256!(5).cmp(&udec256!(10)), Ordering::Less);
assert_eq!(udec256!(10).cmp(&udec256!(5)), Ordering::Greater);
assert_eq!(udec256!(5).cmp(&udec256!(5)), Ordering::Equal);
pub const fn add(self, rhs: Self) -> Self

Calculates self + rhs.

Is internally used by the + operator.

Panics:
debug mode

This method will panic if addition operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

let a = UD256::ONE;
let b = UD256::TWO;

let c = a + b;
assert_eq!(c, udec256!(3));

Panics if overflowed:

use fastnum::*;

let a = UD256::MAX;
let b = UD256::MAX;

let c = a + b;

See more about add and subtract.

pub const fn sub(self, rhs: Self) -> Self

Calculates selfrhs.

Is internally used by the - operator.

Panics:
debug mode

This method will panic if subtract operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

let a = UD256::FIVE;
let b = UD256::TWO;

let c = a - b;
assert_eq!(c, udec256!(3));

Panics if overflowed:

use fastnum::*;

let a = UD256::ZERO;
let b = UD256::ONE;

let c = a - b;

See more about add and subtract.

pub const fn mul(self, rhs: Self) -> Self

Calculates self × rhs.

Is internally used by the * operator.

Panics:
debug mode

This method will panic if multiplication operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

let a = UD256::FIVE;
let b = UD256::TWO;

let c = a * b;
assert_eq!(c, udec256!(10));

Panics if overflowed:

use fastnum::*;

let a = UD256::MAX;
let b = UD256::MAX;

let c = a * b;

See more about multiplication.

pub const fn div(self, rhs: Self) -> Self

Calculates self ÷ rhs.

Is internally used by the / operator.

Panics:
debug mode

This method will panic if divide operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

let a = UD256::FIVE;
let b = UD256::TWO;

let c = a / b;
assert_eq!(c, udec256!(2.5));

Panics if divided by zero:

use fastnum::*;

let a = UD256::ONE;
let b = UD256::ZERO;

let c = a / b;

See more about division.

pub const fn rem(self, rhs: Self) -> Self

Calculates self % rhs.

Is internally used by the % operator.

Panics:
debug mode

This method will panic if reminder operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

let a = UD256::FIVE;
let b = UD256::TWO;

let c = a % b;
assert_eq!(c, udec256!(1));
pub const fn recip(self) -> Self

Takes the reciprocal (inverse) of a number, 1/x.

Panics:
debug mode

This method will panic if reciprocal operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Precision

Since the result of reciprocal is irrational number, it can usually only be computed to some finite precision from a series of increasingly accurate approximations.

The result of this operation is mostly inexact and raises OP_INEXACT signal.

Examples
use fastnum::*;

assert_eq!(udec256!(2).recip(), udec256!(0.5));
pub const fn pow(self, n: Decimal<N>) -> Self

Raise an unsigned decimal number to decimal power.

Using this function is generally slower than using powi for integer exponents or sqrt method for 1/2 exponent.

Panics:
debug mode

This method will panic if power operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

assert_eq!(udec256!(4).pow(dec256!(0.5)), udec256!(2));
assert_eq!(udec256!(8).pow(dec256!(1) / dec256!(3)), udec256!(2));

See more about the power operation.

pub const fn powi(self, n: i32) -> Self

Raise an unsigned decimal number to an integer power.

Using this function is generally faster than using pow

Panics:
debug mode

This method will panic if power operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

assert_eq!(udec256!(2).powi(3), udec256!(8));
assert_eq!(udec256!(9).powi(2), udec256!(81));
assert_eq!(udec256!(1).powi(-2), udec256!(1));
assert_eq!(udec256!(10).powi(20), udec256!(1e20));
assert_eq!(udec256!(4).powi(-2), udec256!(0.0625));

See more about the power operation.

pub const fn sqrt(self) -> Self

Take the square root of the unsigned decimal number.

Square-root can also be calculated by using the power operation (with a second operand of 0.5). The result in that case will not be exact and may not be correctly rounded.

Panics:
debug mode

This method will panic if sqrt operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

assert_eq!(udec128!(4).sqrt(), udec128!(2));
assert_eq!(udec128!(1).sqrt(), udec128!(1));
assert_eq!(udec128!(16).sqrt(), udec128!(4));

See more about the square-root operation.

pub const fn exp(self) -> Self

Returns eself, (the exponential function).

Panics:
debug mode

This method will panic if exponent operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

assert_eq!(udec128!(1).exp(), UD128::E);

See more about the exponential function.

pub const fn ln(self) -> Decimal<N>

Returns the natural logarithm of the decimal number.

Precision

Since the result of natural logarithm is irrational number, it can usually only be computed to some finite precision from a series of increasingly accurate approximations.

The result of this operation is mostly inexact and raises OP_INEXACT signal.

Panics:
debug mode

This method will panic if logarithm operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!(udec256!(2).ln(), D256::LN_2);
assert_eq!(UD256::E.ln(), D256::ONE);

See also:

pub const fn ln_1p(self) -> Decimal<N>

Returns natural logarithm ln(1 + self) more accurately than if the operations were performed separately.

Precision

Since the result of natural logarithm is irrational number, it can usually only be computed to some finite precision from a series of increasingly accurate approximations.

The result of this operation is mostly inexact and raises OP_INEXACT signal.

Panics:
debug mode

This method will panic if logarithm operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!((UD256::E - udec256!(1)).ln_1p(), dec256!(1));

See also:

pub const fn log(self, base: Self) -> Decimal<N>

Returns the logarithm of the decimal number with respect to the given arbitrary base.

Precision

Since the result of logarithm is irrational number, it can usually only be computed to some finite precision from a series of increasingly accurate approximations.

The result of this operation is mostly inexact and raises OP_INEXACT signal.

Panics:
debug mode

This method will panic if logarithm operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!(udec256!(64).log(udec256!(2)), dec256!(6));
assert_eq!(udec256!(27).log(udec256!(3)), dec256!(3));
assert_eq!(udec256!(15625).log(udec256!(5)), dec256!(6));

See also:

pub const fn log2(self) -> Decimal<N>

Returns the binary logarithm of the given decimal number.

Precision

Since the result of logarithm is irrational number, it can usually only be computed to some finite precision from a series of increasingly accurate approximations.

The result of this operation is mostly inexact and raises OP_INEXACT signal.

Panics:
debug mode

This method will panic if logarithm operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!(udec256!(64).log2(), dec256!(6));
assert_eq!(udec256!(32).log2(), dec256!(5));
assert_eq!(udec256!(1024).log2(), dec256!(10));
assert_eq!(udec256!(0.5).log2(), dec256!(-1));
assert_eq!(udec256!(0.25).log2(), dec256!(-2));
assert_eq!(udec256!(10).log2(), D256::LOG2_10);

See also:

pub const fn log10(self) -> Decimal<N>

Returns the decimal logarithm of the given decimal number.

Precision

Since the result of logarithm is irrational number, it can usually only be computed to some finite precision from a series of increasingly accurate approximations.

The result of this operation is mostly inexact and raises OP_INEXACT signal.

Panics:
debug mode

This method will panic if logarithm operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!(udec256!(100).log10(), dec256!(2));
assert_eq!(udec256!(1000).log10(), dec256!(3));
assert_eq!(udec256!(0.1).log10(), dec256!(-1));
assert_eq!(udec256!(0.01).log10(), dec256!(-2));
assert_eq!(udec256!(2).log10(), D256::LOG10_2);

See also:

pub const fn mul_add(self, a: Self, b: Self) -> Self

Fused multiply-add. Computes (self * a) + b with only one rounding error, yielding a more accurate result than an unfused multiply-add.

Panics:
debug mode

This method will panic if multiply-add operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Basic usage:

use fastnum::*;

assert_eq!(udec128!(10.0).mul_add(udec128!(4.0), udec128!(60)), udec128!(100));

See more about the fused multiply-add function.

pub const fn round(self, digits: i16) -> Self

Returns the given decimal number rounded to digits precision after the decimal point, using RoundingMode from it Context.

Panics:
debug mode

This method will panic if round operation (up-scale or down-scale) performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples
use fastnum::{*, decimal::{*, RoundingMode::*}};

let n = udec256!(129.41675);

// Default rounding mode is `HalfUp`
assert_eq!(n.round(2),  udec256!(129.42));

assert_eq!(n.with_rounding_mode(Up).round(2), udec256!(129.42));
assert_eq!(n.with_rounding_mode(Down).round(-1), udec256!(120));
assert_eq!(n.with_rounding_mode(HalfEven).round(4), udec256!(129.4168));

See also:

pub const fn floor(self) -> Self

Returns the largest integer less than or equal to a number.

Examples
use fastnum::*;

assert_eq!(udec256!(3.99).floor(), udec256!(3));
assert_eq!(udec256!(3.0).floor(), udec256!(3.0));
assert_eq!(udec256!(3.01).floor(), udec256!(3));
assert_eq!(udec256!(3.5).floor(), udec256!(3));
assert_eq!(udec256!(4.0).floor(), udec256!(4));
pub const fn ceil(self) -> Self

Finds the nearest integer greater than or equal to x.

Examples
use fastnum::*;

assert_eq!(udec256!(3.01).ceil(), udec256!(4));
assert_eq!(udec256!(3.99).ceil(), udec256!(4));
assert_eq!(udec256!(4.0).ceil(), udec256!(4));
assert_eq!(udec256!(1.0001).ceil(), udec256!(2));
assert_eq!(udec256!(1.00001).ceil(), udec256!(2));
assert_eq!(udec256!(1.000001).ceil(), udec256!(2));
assert_eq!(udec256!(1.00000000000001).ceil(), udec256!(2));
pub const fn rescale(self, new_scale: i16) -> Self

Returns the given decimal number re-scaled to digits precision after the decimal point.

Panics:
debug mode

This method will panic if rescale operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples
use fastnum::{*, decimal::*};

assert_eq!(udec256!(2.17).rescale(3), udec256!(2.170));
assert_eq!(udec256!(2.17).rescale(2), udec256!(2.17));
assert_eq!(udec256!(2.17).rescale(1), udec256!(2.2));
assert_eq!(udec256!(2.17).rescale(0), udec256!(2));
assert_eq!(udec256!(2.17).rescale(-1), udec256!(0));

let ctx = Context::default().without_traps();

assert!(UD256::INFINITY.with_ctx(ctx).rescale(2).is_nan());
assert!(UD256::NAN.with_ctx(ctx).rescale(1).is_nan());

See also:

pub const fn quantize(self, other: Self) -> Self

Returns a value equal to self (rounded), having the exponent of other.

Panics:
debug mode

This method will panic if quantize operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples
use fastnum::{*, decimal::*};

let ctx = Context::default().without_traps();

assert_eq!(udec256!(2.17).quantize(udec256!(0.001)), udec256!(2.170));
assert_eq!(udec256!(2.17).quantize(udec256!(0.01)), udec256!(2.17));
assert_eq!(udec256!(2.17).quantize(udec256!(0.1)), udec256!(2.2));
assert_eq!(udec256!(2.17).quantize(udec256!(1e+0)), udec256!(2));
assert_eq!(udec256!(2.17).quantize(udec256!(1e+1)), udec256!(0));

assert_eq!(UD256::INFINITY.quantize(UD256::INFINITY), UD256::INFINITY);

assert!(udec256!(2).with_ctx(ctx).quantize(UD256::INFINITY).is_nan());

assert_eq!(udec256!(0.1).quantize(udec256!(1)), udec256!(0));
assert_eq!(udec256!(0).quantize(udec256!(1e+5)), udec256!(0E+5));

assert!(udec128!(0.34028).with_ctx(ctx).quantize(udec128!(1e-32765)).is_nan());

assert_eq!(udec256!(217).quantize(udec256!(1e-1)), udec256!(217.0));
assert_eq!(udec256!(217).quantize(udec256!(1e+0)), udec256!(217));
assert_eq!(udec256!(217).quantize(udec256!(1e+1)), udec256!(2.2E+2));
assert_eq!(udec256!(217).quantize(udec256!(1e+2)), udec256!(2E+2));

See also:

pub const fn trunc(self) -> Self

Truncates the decimal number to integral with no fractional portion.

This is a true truncation whereby no rounding is performed. This operation is equivalent to Self::rescale or Self::trunc_with_scale with scale set to 0.

Performance

This operation is typically much faster than Self::rescale

Panics:
debug mode

This method will panic if truncate operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!(udec256!(3.141).trunc(), udec256!(3));
assert_eq!(udec256!(2.9).trunc(), udec256!(2));

let ctx = decimal::Context::default().without_traps();

assert!(UD256::INFINITY.with_ctx(ctx).trunc().is_nan());
assert!(UD256::NAN.with_ctx(ctx).trunc().is_nan());

See also:

pub const fn trunc_with_scale(self, scale: i16) -> Self

Truncates the decimal number to the given number of digits after the decimal point.

This is a true truncation whereby no rounding is performed. This operation is equivalent to Self::rescale.

Performance

This operation is typically much faster than Self::rescale

Panics:
debug mode

This method will panic if truncate operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD256 is used here.

use fastnum::*;

assert_eq!(udec256!(3.141592).trunc_with_scale(2), udec256!(3.14));
assert_eq!(udec256!(3.141592).trunc_with_scale(3), udec256!(3.141));
assert_eq!(udec256!(3.141592).trunc_with_scale(4), udec256!(3.1415));
assert_eq!(udec256!(3.141592).trunc_with_scale(5), udec256!(3.14159));
assert_eq!(udec256!(3.141592).trunc_with_scale(6), udec256!(3.141592));

let ctx = decimal::Context::default().without_traps();

assert!(UD256::INFINITY.with_ctx(ctx).trunc_with_scale(1).is_nan());
assert!(UD256::NAN.with_ctx(ctx).trunc_with_scale(1).is_nan());

See also:

pub const fn is_ok(&self) -> bool

Returns:

pub const fn ok(self) -> Option<Self>

Returns:

pub fn to_scientific_notation(&self) -> String

Create a string of this unsigned decimal in scientific notation.

Examples
use fastnum::udec256;

let n = udec256!(12345678);
assert_eq!(&n.to_scientific_notation(), "1.2345678e7");
pub fn to_engineering_notation(&self) -> String

Create a string of this unsigned decimal in engineering notation.

Engineering notation is scientific notation with the exponent coerced to a multiple of three.

Examples
use fastnum::udec256;

let n = udec256!(12345678);
assert_eq!(&n.to_engineering_notation(), "12.345678e6");
pub const fn to_signed(self) -> Decimal<N>

Converts the given unsigned decimal to a signed decimal number.

Examples
use fastnum::*;

let d = udec256!(1.2345);

assert_eq!(d.to_signed(), dec256!(1.2345));
pub const fn try_from_signed(d: Decimal<N>) -> Result<Self, DecimalError>

Try converts from Decimal to UnsignedDecimal.

Examples
use fastnum::*;

assert_eq!(UD256::try_from_signed(dec256!(1.2345)), Ok(udec256!(1.2345)));
assert!(UD256::try_from_signed(dec256!(-1.2345)).is_err());
pub const fn transmute<const M: usize>(self) -> UnsignedDecimal<M>

👎Deprecated since 0.5.0

Deprecated, use resize instead.

pub const fn resize<const M: usize>(self) -> UnsignedDecimal<M>

Safety resizes the underlying decimal to use M limbs while preserving the numeric value when possible.

This operation can either widen or narrow the internal representation:

  • Widening (M >= N) is lossless: the value is preserved.
  • Narrowing (M < N) may reduce available capacity. In this case the value is rounded according to the current Context and corresponding status flags are set.

Behavior details:

  • Rounding: extra precision is rounded using the active RoundingMode from the current context.
  • Signals: status flags such as Inexact, Rounded, Clamped, Overflow, or Underflow may be raised depending on the operation outcome and context limits.

Note: lossless, no-rounding conversions.

If you need to change width without any rounding:

  • Use crate::Cast for guaranteed-lossless widening (value-preserving by definition).

  • Use crate::TryCast for potential narrowing without rounding; it returns an error if the value does not fit into the target width, thus guaranteeing no silent rounding or truncation.

Performance

This operation is typically much slower than crate::Cast and crate::TryCast transformations.

Panics:
debug mode

This method will panic if resize operation performs with some Exceptional condition and corresponding Signals in the Context is trapped by trap-enabler.

release mode

In release mode panic will not occur and result can be one of Special values(NaN or ±Infinity).

Examples

Please note that this example is shared between decimal types. Which explains why UD64 is used here.

Lossless widening:
use fastnum::*;

let x = udec64!(123.45);

// Increase internal width from 2 to 4 limbs — value is preserved.
let y: UD128 = x.resize();
assert_eq!(y, udec128!(123.45));
assert!(y.is_op_ok());
Narrowing with possible rounding:
use fastnum::*;

let x = udec128!(1.8446744073709551616);

// Reduce width; value may be rounded according to context.
let y: UD64 = x.resize();
// Rounding/precision-loss indicators may be set, depending on capacity and context:
assert_eq!(y, udec64!(1.844674407370955162));
assert!(y.is_op_inexact() && y.is_op_rounded());

See also: