#[repr(u8)]pub enum Sign {
Plus = 0,
Minus = 1,
}Expand description
A Sign represents sign associated with a decimal number.
Variants
Implementations
Sourceimpl Sign
impl Sign
Sourcepub const fn is_negative(self) -> bool
pub const fn is_negative(self) -> bool
Returns true if sign is negative, and false otherwise.
Sourcepub const fn eq(&self, other: &Self) -> bool
pub const fn eq(&self, other: &Self) -> bool
Tests for self and other signs to be equal, and is used by ==
operator.