Signals

Struct Signals 

pub struct Signals(/* private fields */);
Signals

The exceptional conditions are grouped into signals, which can be controlled individually. For each of the signals, the corresponding flag is set to 1 when the signal occurs.

Implementations

Source

impl Signals

pub const EMPTY: Self

By default, no operation signals are raised.

pub const OP_CLAMPED: Self

Raised when the exponent of a result has been altered or constrained to fit the target type.

pub const OP_DIV_BY_ZERO: Self

Raised when a non-zero dividend is divided by zero.

pub const OP_INVALID: Self

Raised when a result would be undefined or impossible.

pub const OP_INEXACT: Self

Raised when a result is not exact (one or more non-zero coefficient digits were discarded during rounding).

pub const OP_OVERFLOW: Self

Raised when the exponent of a result is too large to be represented.

pub const OP_ROUNDED: Self

Raised when a result has been rounded (that is, some zero or non-zero coefficient digits were discarded).

pub const OP_SUBNORMAL: Self

Raised when a result is subnormal (its adjusted exponent is less than Emin), before any rounding.

pub const OP_UNDERFLOW: Self

Raised when a result is both subnormal and inexact.

pub const fn empty() -> Self

Return an empty set of signaling flags.

pub const fn combine(self, other: Self) -> Self

Combines the given signal with another one.

pub const fn intersect(self, other: Self) -> Self

Intersect the given signal with another one.

pub const fn is_empty(&self) -> bool

Is empty