pub struct Signals(/* private fields */);Expand description
Signals
The exceptional conditions are grouped into signals, which can be controlled
individually.
For each of the signals, the corresponding flag is set to 1
when the signal occurs.
Implementations
Sourceimpl Signals
impl Signals
Sourcepub const OP_CLAMPED: Self
pub const OP_CLAMPED: Self
Raised when the exponent of a result has been altered or constrained to fit the target type.
Sourcepub const OP_DIV_BY_ZERO: Self
pub const OP_DIV_BY_ZERO: Self
Raised when a non-zero dividend is divided by zero.
Sourcepub const OP_INVALID: Self
pub const OP_INVALID: Self
Raised when a result would be undefined or impossible.
Sourcepub const OP_INEXACT: Self
pub const OP_INEXACT: Self
Raised when a result is not exact (one or more non-zero coefficient digits were discarded during rounding).
Sourcepub const OP_OVERFLOW: Self
pub const OP_OVERFLOW: Self
Raised when the exponent of a result is too large to be represented.
Sourcepub const OP_ROUNDED: Self
pub const OP_ROUNDED: Self
Raised when a result has been rounded (that is, some zero or non-zero coefficient digits were discarded).
Sourcepub const OP_SUBNORMAL: Self
pub const OP_SUBNORMAL: Self
Raised when a result is subnormal (its adjusted exponent is less than Emin), before any rounding.
Sourcepub const OP_UNDERFLOW: Self
pub const OP_UNDERFLOW: Self
Raised when a result is both subnormal and inexact.