SignalsTraps

Struct SignalsTraps 

pub struct SignalsTraps(/* private fields */);
SignalsTraps

SignalsTraps is a list of set trap enablers for signals. When a signal’s trap enabler is set, the condition causes panic! in debug mode.

Implementations

Source

impl SignalsTraps

pub const fn empty() -> Self

Returns the empty list of signal traps.

pub const fn default() -> Self

Returns the default set of signal traps.

pub const fn set(self, signal: Signals) -> Self

Adds the signal trap for the given signal.